#include "def.h"

extern PAGENO treesearch_page(PAGENO PageNo, char *key);
extern int CompareKeys(char *key, char *word);
extern int FreePage(struct PageHdr *PagePtr);
extern struct PageHdr *FetchPage(PAGENO page);
extern void strtolow(char *str);

typedef struct KeyRecord *RecordPtr;
typedef int (*Visitor)(RecordPtr ptr);

/*
 *  * start from a leaf page and traverse through records
 *   */
void leaf_page_traverse(const PAGENO leaf_page, Visitor v) {
    PAGENO page = leaf_page;
    int stop = 0;
    while ((!stop) && (page != NULLPAGENO)) {
        struct PageHdr *p = FetchPage(page);
        assert(IsLeaf(p) && "not a leaf page");
        RecordPtr i = p->KeyListPtr;
        int record_count = p->NumKeys;
        while ((!stop) && record_count-- > 0) {
            stop = v(i);
            i = i->Next;
        }
        PAGENO next = p->PgNumOfNxtLfPg;
        FreePage(p);
        page = next;
    }
}

static char *lookup_key;
static char *buf;
static int found;

int visit(RecordPtr ptr) {
    char *stored_key = ptr->StoredKey;
    char *stored_prefix = malloc(strlen(stored_key) * sizeof(char));
    strcpy(stored_prefix, stored_key);
    if (strlen(stored_key) > strlen(lookup_key)) {
        stored_prefix[strlen(lookup_key)] = '\0';
    }
    int cmp_result = CompareKeys(lookup_key, stored_prefix);
    if (cmp_result == 1) { /* key > stored */
        strcpy(buf, stored_key);
        found = 1;
    }
    return found == 1;
}

char* nextkey(char *key) {
  int len = strlen(key);
  char last = key[len-1];
  char* ret = malloc((len+1) * sizeof(char));
  strcpy(ret, key);

  if (last != 'z') {
    ret[len-1] = last + 1;
  }

  return ret;
}

int get_rightbracket(char *key, char *result) {
    strtolow(key);
    char* next = nextkey(key);
    const PAGENO page = treesearch_page(ROOT, next);
    buf = result;
    lookup_key = key;
    found = 0;
    leaf_page_traverse(page, &visit);
    free(next);
    return found;
}
