#!/usr/bin/env python
"""
Functional Test Suites for some functionalities in B-tree project.

Assumes that "make load" has already been called

Jiayu Liu - jiayul@andrew.cmu.edu
Feb 4th 2015
Modified by Anna Etzel
Oct 8th 2015
"""
import re
import subprocess
from subprocess import PIPE

BIN_PATH = "./main"

def run_binary(cmd):
    p = subprocess.Popen([BIN_PATH], stdin=PIPE, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate(input=cmd)
    if p.poll():
        print >> stderr, "process did not exit properly"
    if stderr:
        print >> stderr, "stderr", stderr
    return stdout
    
def scan_all_words():
    all_words = run_binary("""
    C
    x
    """)
    return sorted(re.findall(r'^(\w+):\d+', all_words, re.MULTILINE))
    
def uni_gap(lst, size=10):
    return xrange(0, len(lst), len(lst) / (size - 1))
    
def test_search(all_words):
    """
    uniformly sample 10 words to test
    """
    for i in uni_gap(all_words):
        result = run_binary(cmd="""
        s {0}
        x
        """.format(all_words[i]))
        yield "Found the key!" in result
        
def test_rightbracket(all_words):
    """
    uniformly sample 10 words to test
    checks right bracket of first three letters of key
    """
    for i in uni_gap(all_words):
        result = run_binary(cmd="""
        ] {0}
        x
        """.format((all_words[i])[0:3]))
        right = re.findall(r'^(\*?\w+\*?)\s*$', result, re.MULTILINE)[0]
        expected = '*NONE*'
        for j in range(i+1, len(all_words)):
            if not all_words[j].startswith((all_words[i])[0:3]):
                expected = all_words[j]
                break
        if right != expected:
            print "user result", right
            print "expected", expected
        yield right == expected
        
def test_leftbracket(all_words):
    """
    uniformly sample 10 words to test
    checks left bracket of first three letters of key
    """
    for i in uni_gap(all_words):
        result = run_binary(cmd="""
        [ {0}
        x
        """.format((all_words[i])[0:3]))
        left = re.findall(r'^(\*?\w+\*?)\s*$', result, re.MULTILINE)
        if left == []:
            left = ''
        else:
            left = left[0]
        expected = '*NONE*'
        for j in range(i-1, -1, -1):
            if not all_words[j].startswith((all_words[i])[0:3]):
                expected = all_words[j]
                break
        if left != expected:
            print "user result", left
            print "expected", expected
        yield left == expected

def to_score(lst):
    return "score: {0}/{1}".format(sum(filter(lambda x: x is True, lst)), len(lst))

def main():
    all_words = scan_all_words()
    print "the b-tree contains", len(all_words), "words"
    
    print "search on some of then should always return true:"
    search_result = list(test_search(all_words))
    print to_score(search_result) + ' [0.5 points each]'
    
    print "search on right bracket:"
    rightbracket = list(test_rightbracket(all_words))
    print to_score(rightbracket) + ' [1 point each]'
    
    print "search on left bracket:"
    leftbracket = list(test_leftbracket(all_words))
    print to_score(leftbracket) + ' [1 point each]'
    
    
if __name__ == '__main__':
    main()
